package ActiveState::ODE::BuildDB;

use strict;

use ActiveState::DB;

sub NUL() { "\0" }
sub ODE_CHARMAP_SPACE() { 0x01 }
sub ODE_CHARMAP_KRUFT() { 0x02 }
sub ODE_CHARMAP_DIGIT() { 0x04 }
sub ODE_CHARMAP_ALPHA() { 0x08 }

sub build_db {
    my ($re_cb, $cs_space, $cs_kruft, $cs_digit, $cs_alpha, $words, $subs) = @_;
    my $ode_db = ActiveState::DB::Build->new();
    my $charmap = '';
    for (0..255) {
	my $flags = 0;
	$flags |= ODE_CHARMAP_SPACE if index($cs_space, chr($_))!=-1;
	$flags |= ODE_CHARMAP_KRUFT if index($cs_kruft, chr($_))!=-1;
	$flags |= ODE_CHARMAP_DIGIT if index($cs_digit, chr($_))!=-1;
	$flags |= ODE_CHARMAP_ALPHA if index($cs_alpha, chr($_))!=-1;
	$charmap .= pack("C", $flags);
    }
    $ode_db->set("c", $charmap);
    
    my $edb = ActiveState::DB::Build->new;
    my $whash;
    for my $winfo (@$words) {
	my $word = lc $winfo->[0];
	my $starts_with = lc substr($word, 0, 1);
	my $excludes = $winfo->[1];
	my @re;
	for my $ex (@$excludes) {
	    if ((my $re = $ex) =~ s#\A/(.*)/(\w*)\z#$1#) {
		$re = "(?$2)$re" if $2;
		push @re, $re_cb->($re);
	    } else {
		push @re, $re_cb->("(?i)\Q$ex\E");
	    }
	}
	$whash->{$starts_with}{$word} = 1;
	$edb->set($word, join(NUL, @re) . NUL) if @re;
    }
    $ode_db->set("e", $edb->build);

    my $word_db = ActiveState::DB::Build->new;
    for my $w (keys %$whash) {
	my @words;
	for my $wd (keys %{$whash->{$w}}) {
	    push @words, $wd;
	}
	@words = sort {length($b) <=> length ($a)} @words;
	$word_db->set($w, join(NUL, @words) . NUL);
    }
    $ode_db->set('w', $word_db->build);

    my $mightbe_db = ActiveState::DB::Build->new;

    my $mbhash;
    $mbhash->{$_} = $_ for map chr, 0 .. 255;
    for my $c (keys %$subs) {
	my $sub = $subs->{$c};
	for my $pat (@$sub) {
	    $mbhash->{substr($pat,0,1)} .= $c;
	}
    }
    for my $c (keys %$mbhash) {
	$mightbe_db->set($c, $mbhash->{$c});
    }
    
    $ode_db->set('m', $mightbe_db->build);

    my $subs_db = ActiveState::DB::Build->new;

    for my $sub (keys %$subs) {
	$subs_db->set($sub, join(NUL, @{$subs->{$sub}}) . NUL);
    }

    $ode_db->set('s', $subs_db->build);

    return $ode_db;
}


1;
