package ActiveState::SpamEngine::Utils;

use strict;
use base qw(Exporter);

our @EXPORT_OK = qw(unpack_force pack_force unpack_summary pack_summary);

{
    my %PACK = (
        Enabled => 'e',
        w => 'w',
    );
    my %UNPACK = reverse %PACK;

    sub pack_force {
        my $info = shift;
        my @rec;
        for my $key (keys %$info) {
            die "Unknown force key '$key'" unless exists $PACK{$key};
            push @rec, pack("aaa*", $PACK{$key}, '.', $info->{$key});
        }
        join("\0", @rec);
    }

    sub unpack_force {
        my $packed = shift;
        my %info;
        for my $rec (split /\0/, $packed) {
            my ($key, undef, $value) = unpack("aaa*", $rec);
            next unless $UNPACK{$key}; # silently ignore unknown keys
            $info{$UNPACK{$key}} = $value;
        }
        \%info;
    }
}

{
    my %PACK = (
        w => 0,
        Enabled => 1,
        override => 2,
        Desc => 3,
        LongDesc => 4,
    );
    my @UNPACK;
    @UNPACK[values %PACK] = keys %PACK;

    sub pack_summary {
        my $hashref = shift;
        my @v;
        for my $key (keys %$hashref) {
            die "Unknown summary key '$key'" unless exists $PACK{$key};
            $v[$PACK{$key}] = $hashref->{$key};
        }
        for (@v) { $_ = "" unless defined $_ }
        join("\0", @v);
    }

    sub unpack_summary {
        my $line = shift;
        my @v = split /\0/, $line;
        my %v;
        for my $i (0 .. $#v) {
            $v{$UNPACK[$i]} = $v[$i];
        }
        \%v;
    }
}

1;
