package ActiveState::Transformer;
use strict;
use Carp;
use ActiveState::Transformer::Pipeline;

our $VERSION = 1.0;

sub new {
    my $class = shift;
    my $self = bless {}, $class;

    $self->{plugin} = {};
    $self->{plugin_loaded} = {};
    
    return $self;
}

sub load_plugin {
    my ($self, $plugin_spec) = @_;

    my ($plugin_name, $plugin_opts) = _parse_plugin_spec($plugin_spec);

    my $class = "ActiveState::Transformer::Plugin::${plugin_name}";
  
    unless ($self->{plugin_loaded}{$plugin_name}) {
	unless ($::{"ActiveState::"}{"Transformer::"}{"Plugin::"}{"${plugin_name}::"}) {
	    eval "require $class";
	    if ($@) {
		die "Error loading Transformer Plugin '$plugin_name': $@";
	    }
	}
	$self->{plugin_loaded}{$plugin_name} = 1;
    }

    {
	no strict 'refs';
	my $normalize_parameters = "${class}::normalize_parameters";
	if ($plugin_opts) {
	    $plugin_opts = $normalize_parameters->($plugin_opts);
	    croak "Invalid plugin opts '$plugin_opts'" unless $plugin_opts;
	    $plugin_spec = "$plugin_name($plugin_opts)";
	} else {
	    $plugin_spec = "$plugin_name";
	}
	if ($self->{plugin}{$plugin_spec}) {
	    return $self->{plugin}{$plugin_spec};
	}
	$self->{plugin}{$plugin_spec} = $class->new($plugin_opts);
	return $self->{plugin}{$plugin_spec};
    }
}

sub plugin_loaded {
    my $self = shift;
    my $plugin = shift;
    return exists $self->{plugin}{$plugin};
}

sub plugin_list {
    my $self = shift;
    return keys %{$self->{plugin}};
}

sub pipeline {
    my $self = shift;
    my $plugins;
    for my $plugin_spec (@_) {
	my ($plugin_name, $plugin_opts) = _parse_plugin_spec($plugin_spec);
	my $ps;
	if ($plugin_opts) {
	    no strict 'refs';
	    my $normalize_parameters =
	    "ActiveState::Transformer::Plugin::${plugin_name}::normalize_parameters";
	    eval q{ $plugin_opts = $normalize_parameters->($plugin_opts) };
	    $ps = "$plugin_name($plugin_opts)";
	} else {
	    $ps = $plugin_name;
	}
	croak "Could not find plugin spec: $ps" unless $self->{plugin}{$ps};
	push @$plugins, $self->{plugin}{$ps};
    }
    return ActiveState::Transformer::Pipeline->new($plugins);
}

sub _parse_plugin_spec {
    my $plugin_spec = shift;

    my ($plugin_name, $plugin_opts);

    if ($plugin_spec =~ /^\s*([^\(]+?)\s*\((.*)\)\s*$/) {
	$plugin_name = $1;
	$plugin_opts = $2;
    }
    else {
	$plugin_name = $plugin_spec;
    }

    return ($plugin_name, $plugin_opts);
}

1;

