use strict;
package ActiveState::Transformer::Plugin;

use Image::ExifTool;

# Redefine the main tag tables to load in LoadAllTables() (sub-tables are recursed automatically).
@Image::ExifTool::loadAllTables = qw(
    Exif JPEG PNG
);

# Redefine the order to try plugins.  We only want to try JPEG
@Image::ExifTool::fileTypes = qw(
    JPEG PNG
);

%Image::ExifTool::UserDefined = (
    'Image::ExifTool::JFIF::Main' => {
        7 => {
            Name => 'thumb_jfif_x',
            Format => 'int8u',
            Priority => -1,
        },
        8 => {
            Name => 'thumb_jfif_y',
            Format => 'int8u',
            Priority => -1,
        },
    },
);

our $exiftool = undef;

sub new {
    my $class = shift;
    my $self = bless {}, $class;
    
    return $self;
}

sub url_encode {
    my ($self, $text) = @_;

    $text =~ s/([^[:alnum:]])/sprintf("%%%02x", ord($1))/ge;
    return $text;
}

sub get_exiftool {
    my ($self) = @_;

    unless (defined $exiftool)
    {
        $exiftool = new Image::ExifTool();
    }
    return $exiftool;
}

sub normalize {
    my ($self, $metadata, $table) = @_;
    #print STDERR "DEBUG: ActiveState::Transformer::Plugin::normalize(self, metadata=$metadata)\n";

    $table or $table = $self->{normalization_table};
    # The GIF plugin doesn't have a normalization table
    return undef unless $table;

    my @normalized = ();
    my @extra_keys = ();

    for my $pair (sort split(' ', $metadata)) {
        my ($key, $value) = $pair =~ m/^(.*)=([^ ]*)/;
        # print STDERR "DEBUG: PAIR: '$pair', KEY: '$key', VALUE: '$value'\n";
        my $rule = $table->{$key};
        if (defined $rule) {
            if ($rule =~ m/^\d+$/) {
                $value = int($value);
                $value = $value - ($value % $rule);
            } elsif (ref($rule) eq 'ARRAY') {
                # e.g.
                # [small, 5, medium, 500, large]
                my $i = 1;
                for (;$i < @$rule ; $i += 2) {
                    if ($value < $rule->[$i]) {
                        $value = $rule->[$i - 1];
                        last;
                    }
                }
                $value = $rule->[-1] if $i >= @$rule;
            }
            push @normalized, "$key=$value";
        } elsif (exists $table->{$key}) {
                # if undef in the array, just pass the value through, though
                # not in the extra_tags section
                push @normalized, "$key=$value";
            }
        else {  # Record extra metadata keys found
            push @extra_keys, $key unless $self->{no_extra_tags};
        }
    }
    #print STDERR "DEBUG: normalized result = '" . join(" ", @normalized) . "'\n";
    my $extra = '';
    $extra .= " extra_tags=" . join(',', @extra_keys) if int(@extra_keys);
    return join(" ", @normalized) . $extra;
}

1;
