package ActiveState::Transformer::Plugin::CharacterTranslationTable;
use strict;
use base qw(ActiveState::Transformer::Plugin);

# TODO: Split char class range entry parsing into a shared package

sub new  {
    my $self = ActiveState::Transformer::Plugin::new(@_);
    my $class = shift;
    my $opts = shift;
    if ($opts && $opts =~ /^\s*\"(.*?)(?!<\\)\"\s*,\s*\"(.*?)(?!<\\)\"\s*$/) {
	my ($from, $to) = ($1,$2);
	for my $temp ($from, $to) {
	    $temp =~ s/\\x(..)/chr(hex($1))/eg;
	    $temp =~ s/\\\"/"/g;
	}
 
	# Note: the normalize_parameters call should prevent this
	#       and cause the compiler to emit an error, this check
	#       is done here just in case and will continue as best it
	#       can in the event of screwy input.
	my $flen = length($from);
	my $tlen = length($to);
	if ($flen != $tlen) {
	    # crap, compiler didn't catch it! Truncate both.
	    my $min_len = $flen;
	    $min_len = $tlen if ($flen>$tlen);
	    $from = substr($from,0,$min_len);
	    $to = substr($to,0,$min_len);
	}
	# ensure dups are removed, second reference wins if the compiler
	# missed it. (that should be a compiler error), e.g. AA==>>BC is
	# invalid.
	my %tr_hash = ();
	my @fchr_ar = split //, $from;
	my @tchr_ar = split //, $to;
	for (my $i=0;$i<int(@fchr_ar);++$i) {
	    $tr_hash{$fchr_ar[$i]} = $tchr_ar[$i];
	}
	my $tr_lhs = "";
	my $tr_rhs = "";
	for my $key (sort keys %tr_hash) {
	    $tr_lhs .= quotemeta($key);
	    $tr_rhs .= quotemeta($tr_hash{$key});
	}
	my $code = "\$self->{tr_sub} = sub {my \$t = \$_[0]; \$t =~ tr/$tr_lhs/$tr_rhs/; return \$t;}";
	eval $code;
	if ($@) {
	    die "$@";
	}
    }

    return $self;
}

sub hex_escape {
    my $temp = shift;
    $temp =~ s/([\x00-\x1f\x80-\xff])/sprintf("\\x%02X",ord($1))/eg;
    return $temp;
}

# class method
sub normalize_parameters {
    my $opts = shift;

    my ($from_in, $to);
    if ($opts =~ /^\s*\"(.*?)(?!<\\)\"\s*,\s*\"(.*?)(?!<\\)\"\s*$/) {
	($from_in, $to) = ($1,$2);
    }

    unless ($from_in && $to) {
	die "missing from and to. Usage: \"from\",\"to..\"";
    }

    for my $temp ($from_in, $to) {
	$temp =~ s/\\x(..)/chr(hex($1))/eg;
	$temp =~ s/\\\"/"/g;
    }
    
    my $from = $from_in;

    # remove dups from 'from' since you can't logically convert the same
    # character into two different outputs.
    $from =~ s/(.)\1/$1/g;

    if ($from ne $from_in) {
	die "right hand side (from) contains duplicates: \"" .
	hex_escape($from_in) . "\" ne \"" . hex_escapse($from) . "\"";
    }

    my $flen = length($from);
    my $tlen = length($to);
    if ($flen != $tlen) {
	die "from and to must be the same length.";
    }
   
    # escape "'s and hex-ify
    for my $temp ($from, $to) {
	$temp =~ s/\"/\\\"/g;
	$temp = hex_escape($temp);
    }
    
    return "\"$from\",\"$to\"";
}

sub transform {
    my $self = shift;
    return $self->{tr_sub}->(shift) if $self->{tr_sub};
}

1;
